'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var view = require('@codemirror/view');
var language = require('@codemirror/language');

const rgbCallExpRegex = /rgb\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*\)/;
function colorPickersDecorations(view$1) {
    const widgets = [];
    for (const range of view$1.visibleRanges) {
        language.syntaxTree(view$1.state).iterate({
            from: range.from,
            to: range.to,
            enter: (type, from, to) => {
                if (type.name === 'CallExpression') {
                    const callExp = view$1.state.doc.sliceString(from, to);
                    const match = rgbCallExpRegex.exec(callExp);
                    if (!match) {
                        return;
                    }
                    const [_, r, g, b] = match;
                    const color = rgbToHex(Number(r), Number(g), Number(b));
                    const d = view.Decoration.widget({
                        widget: new ColorPickerWidget(color, from, to),
                        side: 1,
                    });
                    widgets.push(d.range(to));
                    return;
                }
                if (type.name === 'ColorLiteral') {
                    const color = toFullHex(view$1.state.doc.sliceString(from, to));
                    const d = view.Decoration.widget({
                        widget: new ColorPickerWidget(color, from, to),
                        side: 1,
                    });
                    widgets.push(d.range(to));
                }
            },
        });
    }
    return view.Decoration.set(widgets);
}
function toFullHex(color) {
    if (color.length === 4) {
        return `#${color[1].repeat(2)}${color[2].repeat(2)}${color[3].repeat(2)}`;
    }
    return color;
}
function rgbComponentToHex(component) {
    const hex = component.toString(16);
    return hex.length == 1 ? '0' + hex : hex;
}
function rgbToHex(r, g, b) {
    return `#${rgbComponentToHex(r)}${rgbComponentToHex(g)}${rgbComponentToHex(b)}`;
}
const wrapperClassName = 'cm-css-color-picker-wrapper';
class ColorPickerWidget extends view.WidgetType {
    constructor(color, from, to) {
        super();
        this.color = color;
        this.from = from;
        this.to = to;
    }
    eq(other) {
        return (other.color === this.color &&
            other.from === this.from &&
            other.to === this.to);
    }
    toDOM() {
        const picker = document.createElement('input');
        picker.dataset.from = this.from.toString();
        picker.dataset.to = this.to.toString();
        picker.type = 'color';
        picker.value = this.color;
        const wrapper = document.createElement('span');
        wrapper.appendChild(picker);
        wrapper.className = wrapperClassName;
        return wrapper;
    }
    ignoreEvent() {
        return false;
    }
}
const colorPicker = view.ViewPlugin.fromClass(class {
    constructor(view) {
        this.decorations = colorPickersDecorations(view);
    }
    update(update) {
        if (update.docChanged || update.viewportChanged) {
            this.decorations = colorPickersDecorations(update.view);
        }
    }
}, {
    decorations: (v) => v.decorations,
    eventHandlers: {
        change: (e, view) => {
            const target = e.target;
            if (target.nodeName !== 'INPUT' ||
                !target.parentElement ||
                !target.parentElement.classList.contains(wrapperClassName)) {
                return false;
            }
            view.dispatch({
                changes: {
                    from: Number(target.dataset.from),
                    to: Number(target.dataset.to),
                    insert: target.value,
                },
            });
            return true;
        },
    },
});

exports.colorPicker = colorPicker;
exports.wrapperClassName = wrapperClassName;
